//Author: David Liu (2010)

#ifndef LRTROLLER_H_
#define LRTROLLER_H_

#include "LRTConnections.h"
#include "LRTDriveEncoders.h"
#include "Relay.h"
#include "Counter.h"
#include "DigitalInput.h"
#include "SafeCANJaguar.h"

class LRTRoller
{
public:
    LRTRoller(SpeedController& esc, LRTDriveEncoders& driveEncoders);

    void ApplyConfig();
    void Init();

    float GetRollerOutput();
    void SetRollerOutput(float speed);

    void SetEnabled(bool enabled);
    bool GetEnabled();

    void SetReverse(bool reverse);
    bool IsReverse();

    float ComputeSpeed(float JoyDelta);
    void ApplyOutput(float JoyDelta);

    bool IsRollerButtonToggling();

protected:
    SpeedController& m_escRaw;
    LRTDriveEncoders& m_driveEncoders;

    float m_rollerAdjustment;
    float m_rollerMultiplier;
    float m_rollerForwardMultiplier;
    float m_rollerReverseMultiplier;
    float m_rollerJoystickGain;
    float m_rollerJoystickTurnGain;

    bool m_isEnabled;
    bool m_isReverse;
    bool m_isRollerButtonToggling;

    string m_prefix;
    const static float kRollerDeadband = 0.1;
};

#endif // LRTROLLER_H_
